@@ -1,6 +1,7 @@
 import numpy as np
 import scipy.sparse as sps
 import warnings
+import time
 
 from ._RiskAnalyzer import _RiskAnalyzer
 from ._solvers import _lp_solver
@@ -13,6 +14,8 @@ class GINIAnalyzer(_RiskAnalyzer):
         * getWeights
         * getRisk
         * getPositions
+        * getRiskComp
+        * getDiversification
         * viewForntiers
         * set_rrate
         * set_mktdata
@@ -21,48 +24,60 @@ class GINIAnalyzer(_RiskAnalyzer):
     """
     def __init__(self, mktdata=None, colname='adjusted', freq='Q', 
                  hlength=1.25, calendar=None, 
-                 rtype='Sharpe', method='ecos'):
+                 rtype='Sharpe', method='ecos', name='GINI'):
         """
         Constructor
 
         Parameters
         ----------
-        `mktdata` : `pandas.DataFrame`, optional
+        `mktdata` : `pandas.DataFrame`, optional;
             Historic daily market data for portfolio components in the format
             returned by `azapy.mktData` function. The default is `None`.
-        `colname` : str, optional
+        `colname` : `str`, optional;
             Name of the price column from mktdata used in the weights 
             calibration. The default is `'adjusted'`.
-        `freq` : str, optional
-            Rate of return horizon in number of business day. it could be 
-            'Q' for quarter or 'M' for month. The default is `'Q'`.
-        `hlength` : float, optional
+        `freq` : `str`, optional;
+            Rate of return horizon. It could be 
+            `'Q'` for quarter or `'M'` for month. The default is `'Q'`.
+        `hlength` : `float`, optional;
             History length in number of years used for calibration. A 
             fractional number will be rounded to an integer number of months.
             The default is `1.25` years.
-        `calendar` : `numpy.busdaycalendar`, optional
+        `calendar` : `numpy.busdaycalendar`, optional;
             Business days calendar. If is it `None` then the calendar will 
-            be set to NYSE business calendar.
+            be set to NYSE business calendar. 
             The default is `None`.
-        `rtype` : str, optional
-            Optimization type. Possible values \n
-                'Risk' : minimization of dispersion (risk) measure for  
-                targeted rate of return. \n
-                'Sharpe' : maximization of generalized Sharpe ratio.\n
-                'Sharpe2' : minimization of the inverse generalized Sharpe 
-                ratio.\n
-                'MinRisk' : minimum dispersion (risk) portfolio.\n
-                'InvNrisk' : optimal portfolio with the same dispersion (risk)
-                value as a benchmark portfolio 
-                (e.g. equal weighted portfolio).\n
-                'RiskAverse' : optimal portfolio for a fixed value of 
-                risk-aversion factor.
-            The default is `'Sharpe'`.
-        `method` : str, optional
+        `rtype` : `str`, optional;
+            Optimization type. Possible values: \n
+                `'Risk'` : optimal-risk portfolio for targeted expected rate of 
+                return.\n
+                `'Sharpe'` : Sharpe-optimal portfolio - maximization solution.\n
+                `'Sharpe2'` : Sharpe-optimal portfolio - minimization solution.\n
+                `'MinRisk'` : minimum risk portfolio.\n
+                `'RiskAverse'` : optimal-risk portfolio for a fixed 
+                risk-aversion factor.\n
+                `'InvNrisk'` : optimal-risk portfolio with the same risk value 
+                as a benchmark portfolio (e.g., same as equal weighted 
+                portfolio).\n
+                `'Diverse'` : optimal-diversified portfolio for targeted
+                expected rate of return (maximum of inverse 1-D).\n
+                `'Diverse2'` : optimal-diversified portfolio for targeted
+                expected rate of return (minmum of 1-D).\n
+                `'MaxDiverse'` : maximum diversified portfolio.\n
+                `'InvNdiverse'` : optimal-diversified portfolio with the same
+                diversification factor as a benchmark portfolio 
+                (e.g., same as equal weighted portfolio).\n
+                `'InvNdrr'` : optima- diversified portfolio with the same 
+                expected rate of return as a benchmark portfolio
+                (e.g., same as equal weighted portfolio).\n
+            The defauls is `'Sharpe'`.
+        `method` : `str`, optional;
             Linear programming numerical method. 
-            Could be: 'ecos', 'highs-ds', 'highs-ipm', 'highs', 
-            'interior-point', 'glpk' and 'cvxopt'.
-            The defualt is `'ecos'`.
+            Could be: `'ecos'`, `'highs-ds'`, `'highs-ipm'`, `'highs'`, 
+            `'interior-point'`, `'glpk'` and `'cvxopt'`.
+            The default is `'ecos'`.
+        `name` : `str`, optional;
+            Object name. The default is `'GINI'`.
             
         Returns
         -------
@@ -70,7 +85,8 @@ class GINIAnalyzer(_RiskAnalyzer):
         """
         self.drate = None
         self.nn2 = None
-        super().__init__(mktdata, colname, freq, hlength, calendar, rtype)
+        super().__init__(mktdata, colname, freq, hlength, calendar, rtype,
+                         name)
         
         self._set_method(method)
         
@@ -85,11 +101,11 @@ class GINIAnalyzer(_RiskAnalyzer):
 
         Parameters
         ----------
-        rrate : pandas.DataFrame
+        rrate : `pandas.DataFrame`
             Market data. It will overwrite the value set by the constructor.
         Returns
         -------
-        None.
+        `None`
         """
         self.nn, self.mm = rrate.shape
         self.muk = rrate.mean()
@@ -161,12 +177,14 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # GINI
@@ -228,12 +246,14 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1., 0.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -298,12 +318,14 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1., 0.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -367,12 +389,14 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1., self.risk]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -431,12 +455,14 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -506,17 +532,19 @@ class GINIAnalyzer(_RiskAnalyzer):
         b_data = [1., 0.]
         
         # calc
+        toc = time.perf_counter()
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
-                        + f"on calibration date {self.rrate.index[-1]}")
-            return np.array([np.nan] * mm)
+           warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                         f"status {res['status']} :: {res['infostring']}")
+           return np.array([np.nan] * mm)
             
         t = res['x'][-1]
         # Gini-Divers
-        self.divers = 1. - res['pcost']
+        self.diverse= 1. - res['pcost']
         # Gini
         self.risk = res['pcost'] / t
         # optimal weights
@@ -529,4 +557,152 @@ class GINIAnalyzer(_RiskAnalyzer):
         self.secondary_risk_comp = np.array([self.risk])
         
         return self.ww
-        
\ No newline at end of file
+    
+    
+    def _risk_inv_diversification(self, d=1):
+        # Order of variables (mm - no of symb, nn - no of observations)
+        # w <- [0 : mm]
+        # s <- [mm : mm + nn2]
+        # t <- mm + nn2
+        # in total dim = mm + nn2 + 1
+        # where nn2 = nn * (nn - 1) / 2
+        mm = self.mm
+        nn2 = self.nn2
+        norm = 1 / self.nn ** 2
+        
+        # build c
+        c_data = list(-self.risk_comp) + [0.] * (nn2 + 1)
+        
+        # build G
+        G_icol = [m for m in range(mm) for _ in range(nn2)] * 2
+        G_irow = list(range(nn2)) * mm + list(range(nn2, 2 * nn2)) * mm
+        G_data = list(self.drate) + list(-self.drate)
+        
+        G_icol += list(range(mm, mm + nn2)) * 2
+        G_irow += list(range(nn2 * 2))
+        G_data += [-1.] * (nn2 * 2)
+        
+        G_icol += list(range(mm)) + [mm + nn2]
+        G_irow += [nn2 * 2] * (mm + 1)
+        G_data += list(-self.muk * d) + [self.mu * d]
+        
+        G_icol += list(range(mm))
+        G_irow += list(range(nn2 * 2 + 1, nn2 * 2 + mm + 1))
+        G_data += [-1.] * mm
+        
+        G_shape = (nn2 * 2 + mm + 1, mm + nn2 + 1)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = [0.] * (nn2 * 2 + mm + 1) 
+        
+        # build A
+        A_icol = list(range(mm, mm + nn2))
+        A_irow = [0] * nn2
+        A_data = [norm] * nn2
+        
+        A_icol += list(range(mm)) + [mm + nn2]
+        A_irow += [1] * (mm + 1)
+        A_data += [1.] * mm + [-1.]
+        
+        A_shape = (2, mm + nn2 + 1)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+        
+        # build b
+        b_data = [1., 0.]
+        
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+ 
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+            
+        t = res['x'][-1]
+        # Gini-Divers
+        self.diverse= 1. + 1. / res['pcost']
+        # Gini
+        self.risk = 1. / t
+        # optimal weights
+        self.ww = np.array(res['x'][:mm] / t)
+        self.ww.shape = mm
+        # rate of return
+        self.RR = np.dot(self.ww, self.muk)
+        # default to risk
+        self.primary_risk_comp = np.array([self.risk])
+        self.secondary_risk_comp = np.array([self.risk])
+        
+        return self.ww
+        
+
+    def _rr_max_diversification(self):
+        # Order of variables (mm - no of symb, nn - no of observations)
+        # w <- [0 : mm]
+        # s <- [mm : mm + nn2]
+        # in total dim = mm + nn2
+        # where nn2 = nn * (nn - 1) / 2
+        mm = self.mm
+        nn2 = self.nn2
+        norm = 1 / self.nn ** 2
+        
+        # build c
+        c_data = list(-self.muk) + [0.] * nn2
+        
+        # build G
+        G_icol = [m for m in range(mm) for _ in range(nn2)] * 2
+        G_irow = list(range(nn2)) * mm + list(range(nn2, 2 * nn2)) * mm
+        G_data = list(self.drate) + list(-self.drate)
+        
+        G_icol += list(range(mm, mm + nn2)) * 2
+        G_irow += list(range(nn2 * 2))
+        G_data += [-1.] * (nn2 * 2)
+        
+        G_icol += list(range(mm))
+        G_irow += list(range(nn2 * 2, nn2 * 2 + mm))
+        G_data += [-1.] * mm
+        
+        G_shape = (nn2 * 2 + mm, mm + nn2)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = [0.] * (nn2 * 2 + mm) 
+        
+        # build A
+        A_icol = list(range(mm)) + list(range(mm, mm + nn2)) + list(range(mm))
+        A_irow = [0] * mm + [1] * (nn2 + mm)
+        A_data = [1.] * mm + [norm] * nn2 + list((self.diverse- 1.) * self.risk_comp)
+        
+        A_shape = (2, mm + nn2)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+        
+        # build b
+        b_data = [1., 0.]
+        
+        # calc
+        toc = time.perf_counter()
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+        self.time_level2 = time.perf_counter() - toc
+ 
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {self.name} on {self.rrate.index[-1]} :: "
+                          f"status {res['status']} :: {res['infostring']}")
+            return np.array([np.nan] * mm)
+            
+        # optimal weights
+        self.ww = np.array(res['x'][:mm])
+        self.ww.shape = mm
+        # rate of return
+        self.RR = -res['pcost']
+        # risk
+        self.risk = norm * np.sum(res['x'][mm:])
+        self.primary_risk_comp = np.array([self.risk])
+        self.secondary_risk_comp = np.array([self.risk])
+        # diversification
+        self.diverse= 1. - self.risk / np.dot(self.ww, self.risk_comp)
+        
+        return self.ww
\ No newline at end of file
