@@ -34,58 +34,58 @@ class Port_ConstW(Port_Rebalanced):
 
         Parameters
         ----------
-        `mktdata` : `pandas.DataFrame`
+        `mktdata` : `pandas.DataFrame`;
             MkT data in the format "symbol", "date", "open", "high", "low",
             "close", "volume", "adjusted", "divd", "split" (e.g. as returned
             by `azapy.readMkT` function).
-        `symb` : list, optional
+        `symb` : `list`, optional;
             List of symbols for the basket components. All symbols MkT data
             should be included in mktdata. If set to `None` the `symb` will be
             set to include all the symbols from `mktdata`. The default
             is `None`.
-        `sdate` : date like, optional
+        `sdate` : date like, optional;
             Start date for historical data. If set to `None` the `sdate` will
             be set to the earliest date in mktdata. The default is `None`.
-        `edate` : date like, optional
+        `edate` : date like, optional;
             End date for historical dates and so the simulation. Must be
             greater than  `sdate`. If it is `None` then `edat`e will be set
             to the latest date in mktdata. The default is `None`.
-        `col_price` : str, optional
+        `col_price` : `str`, optional;
             Column name in the mktdata DataFrame that will be considered
             for portfolio aggregation. The default is `'close'`.
-        `col_divd` :  str, optional
+        `col_divd` :  `str`, optional;
             Column name in the mktdata DataFrame that holds the dividend
             information. The default is `'dvid'`.
-        `col_ref` : str, optional
+        `col_ref` : `str`, optional;
             Column name in the mktdata DataFrame that will be used as a price
             reference for portfolio components. The default is `'adjusted'`.
-        `pname` : str, optional
+        `pname` : `str`, optional;
             The name of the portfolio. The default is `'Port'`.
-        `pcolname` : str, optional
+        `pcolname` : `str`, optional;
             Name of the portfolio price column. If it set to `None` that
             `pcolname=pname`. The default is `None`.
-        `capital` : float, optional
+        `capital` : `float`, optional;
             Initial portfolio Capital in dollars. The default is `100000`.
-        `schedule` : `pandas.DataFrame`, optional
-            Rebalancing schedule, with columns for 'Droll' rolling date and
-            'Dfix' fixing date. If it is `None` than the schedule will be set
+        `schedule` : `pandas.DataFrame`, optional;
+            Rebalancing schedule, with columns for `'Droll'` rolling date and
+            `'Dfix'` fixing date. If it is `None` than the schedule will be set
             using the `freq`, `noffset`, `fixoffset` and `calendar`
             information. The default is `None`.
-        `freq` : str, optional
-            rebalancing frequency. It can be 'Q' for quarterly or 'M' for
+        `freq` : `str`, optional;
+            rebalancing frequency. It can be `'Q'` for quarterly or `'M'` for
             monthly rebalancing, respectively. It is relevant only is schedule
             is `None`. The default is `'Q'`.
-        `noffset` : int, optional
-            Number of business days offset for rebalancing date 'Droll'
+        `noffset` : `int`, optional;
+            Number of business days offset for rebalancing date `'Droll'`
             relative to the end of the period (quart or month). A positive
             value add business days beyond the calendar end of the period while
             a negative value subtracts business days. It is relevant only is
             schedule is `None`. The default is `-3`.
-        `fixoffset` : int, optional
-            Number of business day offset of fixing date 'Dfix' relative to
-            the rebalancing date 'Droll'. It can be 0 or negative. It is
+        `fixoffset` : `int`, optional;
+            Number of business day offset of fixing date `'Dfix'` relative to
+            the rebalancing date `'Droll'`. It can be 0 or negative. It is
             relevant only is schedule is `None`. The default is `-1`.
-        `calendar` : `numpy.busdaycalendar`, optional
+        `calendar` : `numpy.busdaycalendar`, optional;
             Business calendar. If it is `None` then it will be set to NYSE
             business calendar. The default
             vale is `None`.
@@ -114,7 +114,7 @@ class Port_ConstW(Port_Rebalanced):
 
         Parameters
         ----------
-        ww : list (also `numpy.array` or `pandas.Series`), optional
+        `ww` : `list` (also `numpy.array` or `pandas.Series`), optional
 
             List of weights. If it is `pandas.Series` the index should match
             the basket `symb`. Otherwise, the weights are considered in the 
